#! /usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const lodash_1 = require("lodash");
const path_1 = __importDefault(require("path"));
const base = process.cwd();
const readDir = (pathString) => {
    let files = {};
    let styles = {};
    const dirs = fs_1.default.readdirSync(pathString);
    dirs.forEach((dir) => {
        if ([".DS_Store"].includes(dir))
            return true;
        const isDir = fs_1.default.lstatSync(`${pathString}/${dir}`).isDirectory();
        if (isDir) {
            const data = readDir(`${pathString}/${dir}`);
            styles = Object.assign(Object.assign({}, styles), data.styles);
            files = Object.assign(Object.assign({}, files), data.files);
        }
        else {
            const data = readFile(`${pathString}/${dir}`);
            styles = Object.assign(Object.assign({}, styles), data.styles);
            files = Object.assign(Object.assign({}, files), data.files);
        }
        return true;
    });
    return { styles, files };
};
const readFile = (file) => {
    var _a, _b, _c, _d;
    const files = {};
    const styles = {};
    const data = fs_1.default.readFileSync(file, { encoding: "utf8", flag: "r" });
    if (data.includes("StyleSheet.create({")) {
        styles[file] = extractStyleObject(data);
        files[file] =
            ((_a = data.split("StyleSheet.create({")) === null || _a === void 0 ? void 0 : _a[0]) +
                ((_d = (_c = (_b = data.split("StyleSheet.create({")) === null || _b === void 0 ? void 0 : _b[1]) === null || _c === void 0 ? void 0 : _c.split("});")) === null || _d === void 0 ? void 0 : _d[1]);
    }
    else {
        files[file] = data;
    }
    return { styles, files };
};
const extractStyleObject = (data) => {
    var _a, _b, _c;
    const temp = (_c = (_b = (_a = data.split("StyleSheet.create({")) === null || _a === void 0 ? void 0 : _a[1]) === null || _b === void 0 ? void 0 : _b.split("});")) === null || _c === void 0 ? void 0 : _c[0];
    const keys = temp.split(": {").map((text) => {
        var _a;
        const k = text.split(" ");
        const t = text.split(",");
        if (t.length) {
            const l = t[t.length - 2];
            if (l && !((_a = l === null || l === void 0 ? void 0 : l.includes) === null || _a === void 0 ? void 0 : _a.call(l, "}")))
                return undefined;
        }
        return k[k.length - 1];
    });
    return keys.filter((s) => !!s).slice(0, -2);
};
const checkIfStyleImported = (stylePath, styleName, filePath, data) => {
    var _a;
    const ext = `.${styleName.split(".")[1]}`;
    const imports = data
        .split("\n")
        .map((line) => {
        var _a, _b, _c, _d, _e, _f, _g;
        if (line.startsWith("import")) {
            const p = ((_b = (_a = line.split("from ")) === null || _a === void 0 ? void 0 : _a[1]) === null || _b === void 0 ? void 0 : _b.slice(1, -2)) || "";
            return {
                path: p.endsWith(ext) ? p : p + ext,
                name: (_g = (_f = (_e = (_d = (_c = line
                    .split(line.includes("{") ? "{" : "import")) === null || _c === void 0 ? void 0 : _c[1]) === null || _d === void 0 ? void 0 : _d.split(line.includes("{") ? "}" : "from")) === null || _e === void 0 ? void 0 : _e[0]) === null || _f === void 0 ? void 0 : _f.trim()) === null || _g === void 0 ? void 0 : _g.split(", "),
            };
        }
        return undefined;
    })
        .filter((imp) => !!imp)
        .map((p) => {
        var _a, _b;
        return ({
            path: path_1.default.join(base, ((_b = (_a = p === null || p === void 0 ? void 0 : p.path) === null || _a === void 0 ? void 0 : _a.startsWith) === null || _b === void 0 ? void 0 : _b.call(_a, "app"))
                ? ""
                : filePath.split("/").slice(0, -1).join("/"), (p === null || p === void 0 ? void 0 : p.path) || ""),
            name: p === null || p === void 0 ? void 0 : p.name,
        });
    })
        .filter((item) => {
        if (item.path === stylePath) {
            return true;
        }
        return false;
    });
    return (_a = imports === null || imports === void 0 ? void 0 : imports[0]) === null || _a === void 0 ? void 0 : _a.name;
};
const getUsedStyles = (importName, keys, text) => {
    return keys.filter((key) => {
        return text.includes(`${importName}.${key}`);
    });
};
const getDirsUnusedStyles = (dir, files) => {
    const { styles } = readDir(dir);
    const unusedStyles = {};
    Object.keys(styles).forEach((style) => {
        var _a;
        console.log(`Checking unused styles for ${style}`);
        const styleName = (_a = style.split("/")) === null || _a === void 0 ? void 0 : _a[style.split("/").length - 1];
        let usedStyles = [];
        Object.keys(files).forEach((file) => {
            const imports = checkIfStyleImported(path_1.default.join(base, style), styleName, file, files[file]);
            if (imports) {
                imports === null || imports === void 0 ? void 0 : imports.forEach((importName) => {
                    usedStyles = [
                        ...usedStyles,
                        ...getUsedStyles(importName, styles[style], files[file]),
                    ];
                });
            }
        });
        const unused = styles[style].filter((key) => !usedStyles.includes(key));
        if (unused.length)
            unusedStyles[style] = unused;
    });
    return unusedStyles;
};
const remove = (data) => {
    data.forEach((item) => {
        Object.keys(item).forEach((file) => {
            console.log("Removing from " + file);
            let text = fs_1.default.readFileSync(file, { encoding: "utf8", flag: "r" });
            item[file].forEach((key) => {
                let scopeCount = 0;
                let finalText = "";
                text.split("\n").forEach((line) => {
                    var _a, _b;
                    if (line.includes(`${key}: {`) && !scopeCount) {
                        finalText += `${line.split(`${key}: {`)[0]}\n`;
                        if (!((_b = (_a = line.split(`${key}: {`)[1]) === null || _a === void 0 ? void 0 : _a.includes) === null || _b === void 0 ? void 0 : _b.call(_a, "},"))) {
                            scopeCount = 1;
                        }
                    }
                    else if (scopeCount > 0) {
                        if (line.includes("{")) {
                            scopeCount++;
                        }
                        if (line.includes("}")) {
                            scopeCount--;
                        }
                    }
                    else {
                        finalText += `${line}\n`;
                    }
                });
                text = finalText;
            });
            fs_1.default.writeFileSync(file, text);
        });
    });
};
if (process.argv.includes("--remove-json")) {
    const json = (_a = process.argv) === null || _a === void 0 ? void 0 : _a[3];
    const data = require(path_1.default.join(base, json));
    remove(data);
}
else {
    const folders = process.argv.slice(2).filter((d) => d !== "--remove");
    const init = () => {
        let totalUnusedCount = 0;
        const data = [];
        folders.forEach((folder) => {
            const { files } = readDir(base);
            const unUsed = getDirsUnusedStyles(folder, files);
            const count = (0, lodash_1.sum)(Object.keys(unUsed).map((u) => unUsed[u].length));
            console.log(folder, count);
            totalUnusedCount += count;
            data.push(unUsed);
        });
        if (process.argv.includes("--remove")) {
            remove(data);
        }
        else if (data.length)
            fs_1.default.writeFileSync("unused-styles.json", JSON.stringify(data, null, 2));
        console.log(">Total Unused styles: ", totalUnusedCount);
    };
    init();
}
